#!/usr/bin/perl -w
use strict;

my $basename = shift @ARGV or die "usage: $0 <basename>";
my $intype = "";
my $offset = 0;
my $length = 0;
my %sizeof;
open IN, "<$basename.in" or die "can't open $basename.in: $!";
open H, ">$basename.h" or die "can't create $basename.h: $!";
open CC, ">$basename.cc" or die "can't create $basename.cc: $!";
my $had_blurb = 0;
my $ingroup = "";
my $exit = 0;
while(<IN>) {
    chomp;
    if($intype ne "") {
	if(/\};/) {
	    print CC "    { T_END, 0, 0, 0, 0 }\n";
	    print CC "};\n";
	    print H "    enum {\n";
	    print H "       size  = $offset,      ///< Size of this structure in DOS\n";
	    print H "       count = $length       ///< Number of members\n";
            print H "    };\n";
	    print H "    static const TStructureParser parser\[\];\n";
	    $sizeof{$intype} = $offset;
	    $intype = "";
        } elsif(m{^\s*//} || m{^\s*$}) {
            # empty
        } elsif(/^(\s*\/\*.*)/) {
            while (! /\*\//) {
                print H "$_\n";
                chomp ($_ = <IN>);
            }
	} elsif(/^(\s*)((?:const\s*|mutable\s*)*)(\w+)(\s+)([^;]+);(.*)/) {
            my ($ws1, $mod, $type, $ws2, $rest, $com) = ($1, $2, $3, $4, $5, $6);

            # process all "pragmas" here
            my $recoded = ($rest =~ s/\bRECODED\b//);
            my $ignored = ($rest =~ s/\bIGNORED\b//); # for future extension

            # update line
            $rest =~ s/\s+$//;
            $_ = "$ws1$mod$type$ws2$rest;$com";

	    my @values = split /, */, $rest;
            print STDERR "warning: RECODED specified but item is not type char\n"
                if $recoded && $type ne "char";
	    foreach (@values) {
		my ($name, $number);
		$name = $_;
		$number = 1;
		while ($name =~ s/\s*\[(\d+)\]$//) {
		    $number *= $1;
		}
		if ($name !~ /^\w+$/) {
		    print STDERR "warning: name `$name' looks wrong\n";
		}
		if($type eq "int16" || $type eq "int16_t") {
		    print CC "    { T_INT16, $number, offsetof($intype, $name), 0, 0 },\n";
		    $offset += $number * 2;
		    $length++;
                } elsif($type eq "uint16" || $type eq "uint16_t") {
		    print CC "    { T_UINT16, $number, offsetof($intype, $name), 0, 0 },\n";
		    $offset += $number * 2;
		    $length++;
		} elsif($type eq "int32" || $type eq "int32_t") {
		    print CC "    { T_INT32, $number, offsetof($intype, $name), 0, 0 },\n";
		    $offset += $number * 4;
		    $length++;
		} elsif($type eq "uint32" || $type eq "uint32_t") {
		    print CC "    { T_UINT32, $number, offsetof($intype, $name), 0, 0 },\n";
		    $offset += $number * 4;
		    $length++;
		} elsif($type eq "char" || $type eq "uint8" || $type eq "uint8_t") {
                    if ($recoded) {
                        print CC "    { T_RSTRING, $number, offsetof($intype, $name), 0, 0 },\n";
                    } else {
                        print CC "    { T_STRING, $number, offsetof($intype, $name), 0, 0 },\n";
                    }
		    $offset += $number;
		    $length++;
		} elsif(defined $sizeof{$type}) {
		    print CC "    { T_TYPEREF, $number, offsetof($intype, $name), sizeof($type), ${type}::parser },\n";
		    $offset += $sizeof{$type} * $number;
		    $length++;
		} else {
		    print STDERR "can't parse: `$type $_'\n";
                    $exit = 1;
		}
	    }
	} else {
	    print STDERR "can't parse: `$_'\n";
            $exit = 1;
	}
    } else {
        if (/\\ingroup\s+(.*)/) { 
            $ingroup = "  *  \\ingroup $1\n"; 
        } elsif (!$had_blurb) {
            print CC "/**
  *  \\file $basename.cc
  *  \\brief Structure Definitions (Conversion Information)
$ingroup  *
  *  Automatically generated by $0 from $basename.in
  */\n\n";
            print CC "#include \"$basename.h\"\n\n";
            print H "/**
  *  \\file $basename.h
  *  \\brief Structure Definitions (Structures)
$ingroup  *
  *  Automatically generated by $0 from $basename.in
  */\n\n";
            $had_blurb = 1;
        }
	if(/struct (\w+) \{/) {
	    $intype = $1;
	    $offset = 0;
	    $length = 0;
	    print CC "/** Conversion information. Describes how to convert a\n";
	    print CC "    $intype structure to and from DOS. */\n";
	    print CC "const TStructureParser ${intype}::parser\[] = {\n";
	}
    }
    print H $_, "\n";
}
exit $exit;
